/*
 * Copyright (C) 2007 Zeeshan Ali (Khattak) <zeeshanak@gnome.org>
 * Copyright (C) 2006, 2007 OpenedHand Ltd.
 *
 * Author: Zeeshan Ali (Khattak) <zeeshanak@gnome.org>
 *
 * SPDX-License-Identifier: LGPL-2.1-or-later
 */

#ifndef GUPNP_TYPES_PRIVATE_H
#define GUPNP_TYPES_PRIVATE_H

#include <glib-object.h>

G_BEGIN_DECLS

GType
gupnp_data_type_to_gtype (const char *data_type);

G_END_DECLS

#endif /* GUPNP_TYPES_PRIVATE_H */
